html,
body {
  width: 100%;
  margin: 0;
}

html {
  height: 100%;
}

body {
  min-height: 100%;
  padding: 20px;
  box-sizing: border-box;
}

p {
  word-break: break-all;
}

@media (max-width: 500px) {
  h1 {
    margin-top: 80px;
  }
}

.flex {
  display: flex;
  flex-direction: column;
  justify-content: center;
  align-items: center;
}

.block {
  display: block;
  position: relative;
}

.identifier {
  font-size: 18px;
  padding: 10px 10px 10px 5px;
  position: relative;
  width: 300px;
  border: none;
  border-bottom: 1px solid #bfbfbf;
}

input:focus {
  outline: none;
}

.bar {
  content: "";
  height: 2px;
  width: 100%;
  bottom: 0;
  position: absolute;
  background: #00bfb3;
  transition: 0.2s ease transform;
  -moz-transition: 0.2s ease transform;
  -webkit-transition: 0.2s ease transform;
  transform: scaleX(0);
}

.identifier:focus ~ .bar {
  transform: scaleX(1);
}

.btn {
  line-height: 38px;
  background-color: #00bfb3;
  color: #fff;
  white-space: nowrap;
  text-align: center;
  font-size: 14px;
  border: none;
  border-radius: 2px;
  cursor: pointer;
  padding: 5px;
  width: 160px;
  margin: 30px 0;
}

.tips,
.example {
  color: #7b7b7b;
  position: relative;
  align-self: flex-start;
  margin-left: 7.5em;
}

.tips > p:first-child::before {
  position: absolute;
  left: -3em;
  content: "";
  color: #7b7b7b;
}

.example > p:first-child::before {
  position: absolute;
  left: -7.5em;
  content: "Example:";
  color: #7b7b7b;
}

.output pre[class*="language-"] {
  position: relative;
  overflow: auto;

  /* make space  */
  margin: 0 0;
  padding: 3rem 0 1.75rem 1rem;
  border-radius: 10px;
}

.output .copy-button {
  position: absolute;
  right: 1.5rem;
  margin: 1rem 0;
  z-index: 100;
}

.output .copy-button button:hover {
  cursor: pointer;
  background-color: #bcbabb;
}

.output {
  max-width: 100%;
  position: relative;
}

.github-fork-ribbon:before {
  background-color: #333;
}
